/**
 * ESP (Extensions Search Provider)
 * extension.js
 *
 * @author     GdH <G-dH@github.com>
 * @copyright  2024
 * @license    GPL-3.0
 *
 */

'use strict';

const GLib = imports.gi.GLib;
const Main = imports.ui.main;

const ExtensionUtils = imports.misc.extensionUtils;
const MyExtension = ExtensionUtils.getCurrentExtension();
const ExtensionsSearchProviderModule = MyExtension.imports.extensionsSearchProvider.ExtensionsSearchProviderModule;
const Settings = MyExtension.imports.settings;
const Util = MyExtension.imports.util;
const { ExtensionState } = imports.misc.extensionUtils;

function init() {
    ExtensionUtils.initTranslations();
    return new ESP();
}

class ESP {
    enable() {
        this.metadata = MyExtension.metadata;

        this.Me = MyExtension;
        this.Me.gSettings = ExtensionUtils.getSettings(this.Me.metadata['settings-schema']);
        this.Me.Settings = Settings;
        this.Me.Util = Util;
        this.Me._ = imports.gettext.domain(this.Me.metadata['gettext-domain']).gettext;

        this.Me.opt = new this.Me.Settings.Options(this.Me);

        this._esp = new ExtensionsSearchProviderModule(this.Me);
        this._esp.update();

        console.debug(`${this.metadata.name}: enabled`);
    }

    disable() {
        if (this._esp) {
            this._esp.update(true);
            this._esp.cleanGlobals();
            this._esp = null;
        }
        if (this.Me) {
            this.Me.opt.destroy();
            this.Me.opt = null;
            this.Me.Util.cleanGlobals();
            this.Me = null;
        }

        console.debug(`${this.metadata.name}: disabled`);
    }
}
